import {html, LitElement, unsafeCSS} from 'lit-element';
import styles from './styles.css';
import {spread} from '@open-wc/lit-helpers';
import {spreadProps} from '@open-wc/lit-helpers';
import {FormCmpMixin} from '../../mixins/FormCmpMixin.js';
import { classMap } from 'lit-html/directives/class-map';

class CWTextField extends FormCmpMixin(LitElement) {
  static get styles() {
    return unsafeCSS(styles);
  }

  static get properties() {
    return {
      label: { type: String, reflect: true },
      placeholder: { type: String, reflect: true },
      description: { type: String, reflect: true },
    };
  }

  constructor() {
    super();
    this.label = '';
    this.placeholder = '';
  }

  _onChangeValue({target: {value}}) {
    this.value = value;
  }

  validate() {
    if (this.required && !this.value)
      return`value required`;
    return true;
  }

  render() {
    const {label, description, placeholder, value, readonly, required, name, touched} = this;
    const inputAttrs = {placeholder, required, name};
    const error = this.validate();
    const isInvalid = touched && typeof error === 'string';
    return html`
      <div class="label-container">
        <label for="value">${label}</label>
      </div>
      <div class="value-container ${classMap({error: isInvalid})}">
        ${readonly
        ? html`<span>${value}</span>`
        : html`
              <input
                id="value"
                type="text"
                @change=${this._onChangeValue}
                .value=${value}
                ...=${spread(inputAttrs)}
              />
            `}
        ${description && html` <div class="description">${description}</div> `}
      </div>
      ${isInvalid ? html`
          <div class="error-container">
              ${error}
          </div>`
      : ``}
    `
  }
}

window.customElements.define('cw-text-field', CWTextField);
