const path = require('path');
const fs = require('fs');

module.exports = {mdxReporter: function mdxReporter()
{
    return {
        onTestRunFinished({sessions, testCoverage}) {
            const coverageSummary = testCoverage.summary;
            const getTests = (tests) => {
                return tests.map((test)=>{
                    return `|${test.name}|${test.passed ? '<font color="#32cd32">Ok</font>' : '<font color="red">'+ test.error.message + '</font>'}|`
                }).join('\n')
            }
            const getResults = (suites) => {
                return suites.map((suite)=>{
                    return `                     
|name|result|
|------------|-------------|
${getTests(suite.tests)};`
                }).join('\n');
            }
            
            const filepath = path.resolve(process.cwd(), './src/stories/testresults.stories.mdx');
            const dir = path.dirname(filepath);
            const template = `<Meta title="Test/Results" />

## Test results:
#### Coverage:
>**lines** count:${coverageSummary.lines.total}, coverage:${coverageSummary.lines.pct}%  
**statements** count:${coverageSummary.statements.total}, coverage:${coverageSummary.statements.pct}%  
**functions** count:${coverageSummary.functions.total}, coverage:${coverageSummary.functions.pct}%  
**branches** count:${coverageSummary.branches.total}, coverage:${coverageSummary.branches.pct}%  

#### Tests:
            ${sessions.map(session=>`
 - ${path.parse(session.testFile).base}  
      ${getResults(session.testResults.suites)}`).join('\n')}
            `
            if (!fs.existsSync(dir)) fs.mkdirSync(dir, {recursive: true});
            fs.writeFileSync(filepath, template);
        }
    };
}
}